/**  
* <p>Title: QyzxAuthController.java</p>  
* <p>Description: </p>  
* @author dsc  
* @date 2020年5月19日  
* @version 1.0  
*/
package cn.timer.api.controller.qyzx;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections4.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import cn.timer.api.aspect.lang.annotation.Log;
import cn.timer.api.aspect.lang.enums.BusinessType;
import cn.timer.api.bean.qyzx.QyzxEmpEntAsso;
import cn.timer.api.bean.qyzx.auth.QyzxAuthAccount;
import cn.timer.api.bean.qyzx.auth.QyzxAuthChild;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enums.SysRoleType;
import cn.timer.api.dao.yggl.YgglMainEmpMapper;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>
 * Title: Auth.java
 * </p>
 * <p>
 * Description:
 * </p>
 * 
 * @author dsc
 * @date 2020年5月19日
 * @version 1.0
 */
@Transactional
@RestController
@Api(tags = "4.5企业中心(账号权限(新))")
@RequestMapping(value = "/Auth", produces = { "application/json" })
public class Auth {

	private QyzxEmpEntAsso selectMain(Integer orgCode, Integer empNum) {
		return QyzxEmpEntAsso.builder().build()
				.selectOne(new LambdaQueryWrapper<QyzxEmpEntAsso>().eq(QyzxEmpEntAsso::getOrgCode, orgCode)
						.eq(QyzxEmpEntAsso::getEmpNum, empNum)
						.eq(QyzxEmpEntAsso::getUserType, SysRoleType.U_TYPE_ADMIN.getType()));
	}

	@Transactional
	@PostMapping(value = "/addChildAccount")
	@ApiOperation(value = "新增子账号", httpMethod = "POST", notes = "接口发布说明")
	@Log(title = "新增-子账号", businessType = BusinessType.INSERT)
	public Result<QyzxAuthChild> addChildAccount(@CurrentUser UserBean userBean, @RequestParam Integer childEmpNum) {
		Integer orgCode = userBean.getOrgCode();
		Integer empNum = userBean.getEmpNum();
		QyzxEmpEntAsso qyzxEmpEntAsso = selectMain(orgCode, empNum);
		System.err.println(qyzxEmpEntAsso);
		if (qyzxEmpEntAsso != null) {
			if (QyzxEmpEntAsso.builder().build()
					.selectCount(new LambdaQueryWrapper<QyzxEmpEntAsso>().eq(QyzxEmpEntAsso::getOrgCode, orgCode)
							.eq(QyzxEmpEntAsso::getUserType, SysRoleType.U_TYPE_C_ADMIN.getType())) >= qyzxEmpEntAsso
									.getMaxNum())
				return ResultUtil.error("子账号数量超过上限");
			if (QyzxEmpEntAsso.builder().empNum(childEmpNum).orgCode(orgCode)
					.userType(SysRoleType.U_TYPE_C_ADMIN.getType()).build().insert())
				return ResultUtil.success("添加成功");
		}
		return ResultUtil.error("暂无添加权限");
	}

	@Transactional
	@DeleteMapping(value = "/delChildAccount/{childEmpNum}")
	@ApiOperation(value = "删除子账号", httpMethod = "DELETE", notes = "接口发布说明")
	@Log(title = "删除-子账号", businessType = BusinessType.DELETE)
	public Result<String> delChildAccount(@CurrentUser UserBean userBean, @PathVariable Integer childEmpNum) {
		Integer orgCode = userBean.getOrgCode();
		Integer empNum = userBean.getEmpNum();
		if (selectMain(orgCode, empNum) != null) {
			if (QyzxEmpEntAsso.builder().build()
					.delete(new LambdaQueryWrapper<QyzxEmpEntAsso>().eq(QyzxEmpEntAsso::getOrgCode, orgCode)
							.eq(QyzxEmpEntAsso::getEmpNum, childEmpNum)
							.eq(QyzxEmpEntAsso::getUserType, SysRoleType.U_TYPE_C_ADMIN.getType())))
				return ResultUtil.success("删除成功");
		}
		return ResultUtil.error("暂无删除权限");
	}

//	@PostMapping(value = "/modifyMaxChild")
//	@ApiOperation(value = "修改子账号最大数量", httpMethod = "POST", notes = "接口发布说明")
//	@Log(title = "修改-子账号最大数量", businessType = BusinessType.UPDATE)
//	public Result<QyzxAuthChild> modifyMaxChild(@CurrentUser UserBean userBean, @RequestParam Integer maxNum) {
//		Integer orgCode = userBean.getOrgCode();
//		Integer empNum = userBean.getEmpNum();
//		QyzxEmpEntAsso qyzxEmpEntAsso = selectMain(orgCode, empNum);
//		if (qyzxEmpEntAsso != null && qyzxEmpEntAsso.getUserType() == 99) { // 99-后台管理员
//			if (QyzxEmpEntAsso.builder().maxNum(maxNum).build().update(
//					new QueryWrapper<QyzxEmpEntAsso>().lambda().eq(QyzxEmpEntAsso::getOrgCode, userBean.getOrgCode())))
//				return ResultUtil.success("修改成功");
//			return ResultUtil.error("修改失败");
//		}
//		return ResultUtil.error("暂无修改权限");
//	}

	@GetMapping(value = "/getAll")
	@ApiOperation(value = "获取全部账号信息", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<YgglMainEmp>> getAll(@CurrentUser UserBean userBean) {
		List<Integer> roles = new ArrayList<Integer>();
		roles.add(SysRoleType.U_TYPE_ADMIN.getType());
		roles.add(SysRoleType.U_TYPE_C_ADMIN.getType());
		List<QyzxEmpEntAsso> list = QyzxEmpEntAsso.builder().build().selectList(
				new LambdaQueryWrapper<QyzxEmpEntAsso>().eq(QyzxEmpEntAsso::getOrgCode, userBean.getOrgCode())
						.in(QyzxEmpEntAsso::getUserType, roles).select(QyzxEmpEntAsso::getEmpNum));
		List<YgglMainEmp> ygs = new ArrayList<YgglMainEmp>();
		List<Integer> empNums = list.stream().map(QyzxEmpEntAsso::getEmpNum).collect(Collectors.toList());
		for (Integer empNum : empNums) {
			YgglMainEmp yg = YgglMainEmp.builder().build()
					.selectOne(new LambdaQueryWrapper<YgglMainEmp>().eq(YgglMainEmp::getOrgCode, userBean.getOrgCode())
							.eq(YgglMainEmp::getEmpNum, empNum)
							.select(YgglMainEmp::getName, YgglMainEmp::getPhone, YgglMainEmp::getEmpNum));
			if (yg != null)
				ygs.add(yg);
		}
		return ResultUtil.data(ygs, "查询成功");
	}

}
